@@ -13,8 +13,6 @@
 ******************************************************************************/
 
 #include <stdlib.h>
-#include <stdbool.h>
-#include <unistd.h>
 #include <string.h>
 #include <stdio.h>
 #include <sys/stat.h>
@@ -29,6 +27,7 @@
 #include "bkError.h"
 #include "bkLink.h"
 #include "bkMisc.h"
+#include "bkIoWrappers.h"
 
 /* numbers as recorded on image */
 #define VDTYPE_BOOT 0
@@ -44,10 +43,10 @@
 * returns number of chars appended
 * destMaxLen doesn't include '\0'
 * if maxSrcLen is -1 tries to copy all of it */
-int appendStringIfHaveRoom(char* dest, char* src, int destMaxLen, 
-                           int destCharsAlreadyUsed, int maxSrcLen)
+size_t appendStringIfHaveRoom(char* dest, const char* src, size_t destMaxLen, 
+                              size_t destCharsAlreadyUsed, int maxSrcLen)
 {
-    int srcLen;
+    size_t srcLen;
     
     if(maxSrcLen == -1)
         srcLen = strlen(src);
@@ -68,30 +67,32 @@ int appendStringIfHaveRoom(char* dest, char* src, int destMaxLen,
 * */
 int bk_open_image(VolInfo* volInfo, const char* filename)
 {
-    int rc;
-    struct stat statStruct;
+    size_t len;
     
-    volInfo->imageForReading = open(filename, O_RDONLY);
+    volInfo->imageForReading = open(filename, O_RDONLY, 0);
     if(volInfo->imageForReading == -1)
     {
         volInfo->imageForReading = 0;
         return BKERROR_OPEN_READ_FAILED;
     }
     
+    int rc;
+    BkStatStruct statStruct;
+    
     /* record inode number */
-    rc = stat(filename, &statStruct);
+    rc = bkStat(filename, &statStruct);
     if(rc == -1)
         return BKERROR_STAT_FAILED;
-
+    
     volInfo->imageForReadingInode = statStruct.st_ino;
     
     /* skip the first 150 sectors if the image is an NRG */
-    int len = strlen(filename);
+    len = strlen(filename);
     if( (filename[len - 3] == 'N' || filename[len - 3] == 'n') &&
         (filename[len - 2] == 'R' || filename[len - 2] == 'r') &&
         (filename[len - 1] == 'G' || filename[len - 1] == 'g') )
     {
-        lseek(volInfo->imageForReading, NBYTES_LOGICAL_BLOCK * 16, SEEK_SET);
+        readSeekSet(volInfo, NBYTES_LOGICAL_BLOCK * 16, SEEK_SET);
     }
     
     return 1;
@@ -108,9 +109,9 @@ int bk_read_dir_tree(VolInfo* volInfo, int filenameType,
     volInfo->progressFunction = progressFunction;
     
     if(filenameType == FNTYPE_ROCKRIDGE || filenameType == FNTYPE_9660)
-        lseek(volInfo->imageForReading, volInfo->pRootDrOffset, SEEK_SET);
+        readSeekSet(volInfo, volInfo->pRootDrOffset, SEEK_SET);
     else /* if(filenameType == FNTYPE_JOLIET) */
-        lseek(volInfo->imageForReading, volInfo->sRootDrOffset, SEEK_SET);
+        readSeekSet(volInfo, volInfo->sRootDrOffset, SEEK_SET);
     
     return readDir(volInfo, &(volInfo->dirTree), 
                    filenameType, keepPosixPermissions);
@@ -128,6 +129,12 @@ int bk_read_vol_info(VolInfo* volInfo)
     bool haveMorePvd; /* to skip extra pvds */
     unsigned char escapeSequence[3]; /* only interested in a joliet sequence */
     char timeString[17]; /* for creation time */
+    bk_off_t locationOfNextDescriptor;
+    unsigned bootCatalogLocation; /* logical sector number */
+    char elToritoSig[24];
+    unsigned char bootMediaType;
+    unsigned short bootRecordSize;
+    unsigned bootRecordSectorNumber;
     
     /* vars for checking rockridge */
     unsigned realRootLoc; /* location of the root dr inside root dir */
@@ -141,11 +148,11 @@ int bk_read_vol_info(VolInfo* volInfo)
     volInfo->sRootDrOffset = 0;
     
     /* skip system area */
-    lseek(volInfo->imageForReading, NLS_SYSTEM_AREA * NBYTES_LOGICAL_BLOCK, SEEK_SET);
+    readSeekSet(volInfo, NLS_SYSTEM_AREA * NBYTES_LOGICAL_BLOCK, SEEK_SET);
     
     /* READ PVD */
     /* make sure pvd exists */
-    rc = read711(volInfo->imageForReading, &vdType);
+    rc = read711(volInfo, &vdType);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
@@ -153,30 +160,30 @@ int bk_read_vol_info(VolInfo* volInfo)
     if(vdType != VDTYPE_PRIMARY)
         return BKERROR_VD_NOT_PRIMARY;
     
-    lseek(volInfo->imageForReading, 39, SEEK_CUR);
+    readSeekSet(volInfo, 39, SEEK_CUR);
     
-    rc = read(volInfo->imageForReading, volInfo->volId, 32);
+    rc = readRead(volInfo, volInfo->volId, 32);
     if(rc != 32)
         return BKERROR_READ_GENERIC;
     volInfo->volId[32] = '\0';
     stripSpacesFromEndOfString(volInfo->volId);
     
-    lseek(volInfo->imageForReading, 84, SEEK_CUR);
+    readSeekSet(volInfo, 84, SEEK_CUR);
     
     /* am now at root dr */
-    volInfo->pRootDrOffset = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+    volInfo->pRootDrOffset = readSeekTell(volInfo);
     
     /* SEE if rockridge exists */
-    lseek(volInfo->imageForReading, 2, SEEK_CUR);
+    readSeekSet(volInfo, 2, SEEK_CUR);
     
-    rc = read733(volInfo->imageForReading, &realRootLoc);
+    rc = read733(volInfo, &realRootLoc);
     if(rc != 8)
         return BKERROR_READ_GENERIC;
     realRootLoc *= NBYTES_LOGICAL_BLOCK;
     
-    lseek(volInfo->imageForReading, realRootLoc, SEEK_SET);
+    readSeekSet(volInfo, realRootLoc, SEEK_SET);
     
-    rc = read711(volInfo->imageForReading, &recordLen);
+    rc = read711(volInfo, &recordLen);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
@@ -184,11 +191,11 @@ int bk_read_vol_info(VolInfo* volInfo)
     /* a minimum root with SP su field */
     {
         /* root dr has filename length of 1 */
-        lseek(volInfo->imageForReading, 33, SEEK_CUR);
+        readSeekSet(volInfo, 33, SEEK_CUR);
         
         /* a rockridge root dr has an SP su entry here */
         
-        rc = read(volInfo->imageForReading, &sPsUentry, 7);
+        rc = readRead(volInfo, &sPsUentry, 7);
         if(rc != 7)
             return BKERROR_READ_GENERIC;
         
@@ -202,73 +209,69 @@ int bk_read_vol_info(VolInfo* volInfo)
     }
     
     /* go back to where it was before trying rockridge */
-    lseek(volInfo->imageForReading, volInfo->pRootDrOffset, SEEK_SET);
+    readSeekSet(volInfo, volInfo->pRootDrOffset, SEEK_SET);
     /* END SEE if rockridge exists */
     
-    lseek(volInfo->imageForReading, 162, SEEK_CUR);
+    readSeekSet(volInfo, 162, SEEK_CUR);
     
-    rc = read(volInfo->imageForReading, volInfo->publisher, 128);
+    rc = readRead(volInfo, volInfo->publisher, 128);
     if(rc != 128)
         return BKERROR_READ_GENERIC;
     volInfo->publisher[128] = '\0';
     stripSpacesFromEndOfString(volInfo->publisher);
     
-    rc = read(volInfo->imageForReading, volInfo->dataPreparer, 128);
+    rc = readRead(volInfo, volInfo->dataPreparer, 128);
     if(rc != 128)
         return BKERROR_READ_GENERIC;
     volInfo->dataPreparer[128] = '\0';
     stripSpacesFromEndOfString(volInfo->dataPreparer);
     
-    lseek(volInfo->imageForReading, 239, SEEK_CUR);
+    readSeekSet(volInfo, 239, SEEK_CUR);
     
-    rc = read(volInfo->imageForReading, timeString, 17);
+    rc = readRead(volInfo, timeString, 17);
     if(rc != 17)
         return BKERROR_READ_GENERIC;
     
     longStringToEpoch(timeString, &(volInfo->creationTime));
     
     /* skip the rest of the extent */
-    lseek(volInfo->imageForReading, 1218, SEEK_CUR);
+    readSeekSet(volInfo, 1218, SEEK_CUR);
     /* END READ PVD */
     
     /* SKIP all extra copies of pvd */
     haveMorePvd = true;
     while(haveMorePvd)
     {
-        rc = read711(volInfo->imageForReading, &vdType);
+        rc = read711(volInfo, &vdType);
         if(rc != 1)
             return BKERROR_READ_GENERIC;
         
         if(vdType == VDTYPE_PRIMARY)
         {
-            lseek(volInfo->imageForReading, 2047, SEEK_CUR);
+            readSeekSet(volInfo, 2047, SEEK_CUR);
         }
         else
         {
-            lseek(volInfo->imageForReading, -1, SEEK_CUR);
+            readSeekSet(volInfo, -1, SEEK_CUR);
             haveMorePvd = false;
         }
     }
     /* END SKIP all extra copies of pvd */
     
     /* TRY read boot record */
-    off_t locationOfNextDescriptor;
-    unsigned bootCatalogLocation; /* logical sector number */
-    char elToritoSig[24];
-    unsigned char bootMediaType;
+   
+    locationOfNextDescriptor = readSeekTell(volInfo) + 2048;
     
-    locationOfNextDescriptor = lseek(volInfo->imageForReading, 0, SEEK_CUR) + 2048;
-    
-    rc = read711(volInfo->imageForReading, &vdType);
+    rc = read711(volInfo, &vdType);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
     if(vdType == VDTYPE_BOOT)
     {
         
-        lseek(volInfo->imageForReading, 6, SEEK_CUR);
+        readSeekSet(volInfo, 6, SEEK_CUR);
         
-        rc = read(volInfo->imageForReading, elToritoSig, 24);
+        rc = readRead(volInfo, elToritoSig, 24);
         if(rc != 24)
             return BKERROR_READ_GENERIC;
         elToritoSig[23] = '\0'; /* just in case */
@@ -276,21 +279,21 @@ int bk_read_vol_info(VolInfo* volInfo)
         if(strcmp(elToritoSig, "EL TORITO SPECIFICATION") == 0)
         /* el torito confirmed */
         {
-            lseek(volInfo->imageForReading, 40, SEEK_CUR);
+            readSeekSet(volInfo, 40, SEEK_CUR);
             
-            rc = read731(volInfo->imageForReading, &bootCatalogLocation);
+            rc = read731(volInfo, &bootCatalogLocation);
             if(rc != 4)
                 return BKERROR_READ_GENERIC;
             
-            lseek(volInfo->imageForReading, bootCatalogLocation * NBYTES_LOGICAL_BLOCK, SEEK_SET);
+            readSeekSet(volInfo, bootCatalogLocation * NBYTES_LOGICAL_BLOCK, SEEK_SET);
             
             /* skip validation entry */
-            lseek(volInfo->imageForReading, 32, SEEK_CUR);
+            readSeekSet(volInfo, 32, SEEK_CUR);
             
             /* skip boot indicator */
-            lseek(volInfo->imageForReading, 1, SEEK_CUR);
+            readSeekSet(volInfo, 1, SEEK_CUR);
             
-            rc = read(volInfo->imageForReading, &bootMediaType, 1);
+            rc = readRead(volInfo, &bootMediaType, 1);
             if(rc != 1)
                 return BKERROR_READ_GENERIC;
             if(bootMediaType == 0)
@@ -303,22 +306,21 @@ int bk_read_vol_info(VolInfo* volInfo)
                 volInfo->bootMediaType = BOOT_MEDIA_2_88_FLOPPY;
             else if(bootMediaType == 4)
             {
-                //!! print warning
+                /* !! print warning */
                 printf("hard disk boot emulation not supported\n");
                 volInfo->bootMediaType = BOOT_MEDIA_NONE;
             }
             else
             {
-                //!! print warning
+                /* !! print warning */
                 printf("unknown boot media type on iso\n");
                 volInfo->bootMediaType = BOOT_MEDIA_NONE;
             }
             
             /* skip load segment, system type and unused byte */
-            lseek(volInfo->imageForReading, 4, SEEK_CUR);
+            readSeekSet(volInfo, 4, SEEK_CUR);
             
-            unsigned short bootRecordSize;
-            rc = read721(volInfo->imageForReading, &bootRecordSize);
+            rc = read721(volInfo, &bootRecordSize);
             if(rc != 2)
                 return BKERROR_READ_GENERIC;
             volInfo->bootRecordSize = bootRecordSize;
@@ -334,39 +336,40 @@ int bk_read_vol_info(VolInfo* volInfo)
             
             volInfo->bootRecordIsOnImage = true;
             
-            unsigned bootRecordSectorNumber;
-            rc = read731(volInfo->imageForReading, &bootRecordSectorNumber);
+            rc = read731(volInfo, &bootRecordSectorNumber);
             if(rc != 4)
                 return BKERROR_READ_GENERIC;
             volInfo->bootRecordOffset = bootRecordSectorNumber * 
                                         NBYTES_LOGICAL_BLOCK;
         }
         else
-            //!! print warning
+            /* !! print warning */
             printf("err, boot record not el torito\n");
         
         /* go to the sector after the boot record */
-        lseek(volInfo->imageForReading, locationOfNextDescriptor, SEEK_SET);
+        readSeekSet(volInfo, locationOfNextDescriptor, SEEK_SET);
     }
     else
     /* not boot record */
     {
         /* go back */
-        lseek(volInfo->imageForReading, -1, SEEK_CUR);
+        readSeekSet(volInfo, -1, SEEK_CUR);
     }
     /* END TRY read boot record */
     
     /* TRY read svd */
-    rc = read711(volInfo->imageForReading, &vdType);
+    rc = read711(volInfo, &vdType);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
     if(vdType == VDTYPE_SUPPLEMENTARY)
     /* make sure it's joliet (by escape sequence) */
     {
-        lseek(volInfo->imageForReading, 87, SEEK_CUR);
+        readSeekSet(volInfo, 87, SEEK_CUR);
         
-        read(volInfo->imageForReading, escapeSequence, 3);
+        rc = readRead(volInfo, escapeSequence, 3);
+        if(rc != 3)
+            return BKERROR_READ_GENERIC;
         
         if( (escapeSequence[0] == 0x25 && escapeSequence[1] == 0x2F &&
              escapeSequence[2] == 0x40) ||
@@ -376,9 +379,9 @@ int bk_read_vol_info(VolInfo* volInfo)
              escapeSequence[2] == 0x45) )
         /* is indeed joliet */
         {
-            lseek(volInfo->imageForReading, 65, SEEK_CUR);
+            readSeekSet(volInfo, 65, SEEK_CUR);
             
-            volInfo->sRootDrOffset = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+            volInfo->sRootDrOffset = readSeekTell(volInfo);
             
             volInfo->filenameTypes |= FNTYPE_JOLIET;
         }
@@ -392,21 +395,21 @@ int bk_read_vol_info(VolInfo* volInfo)
 * dirDrFollows()
 * checks whether the next directory record is for a directory (not a file)
 * */
-bool dirDrFollows(int image)
+bool dirDrFollows(VolInfo* volInfo)
 {
     unsigned char fileFlags;
-    off_t origPos;
+    bk_off_t origPos;
     int rc;
     
-    origPos = lseek(image, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
     
-    lseek(image, 25, SEEK_CUR);
+    readSeekSet(volInfo, 25, SEEK_CUR);
     
-    rc = read711(image, &fileFlags);
+    rc = read711(volInfo, &fileFlags);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
-    lseek(image, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     if((fileFlags >> 1 & 1) == 1)
         return true;
@@ -421,19 +424,19 @@ bool dirDrFollows(int image)
 * If the next byte is zero returns false otherwise true
 * File position remains unchanged
 * Also returns false on read error */
-bool haveNextRecordInSector(int image)
+bool haveNextRecordInSector(VolInfo* volInfo)
 {
-    off_t origPos;
+    bk_off_t origPos;
     char testByte;
     int rc;
     
-    origPos = lseek(image, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
     
-    rc = read(image, &testByte, 1);
+    rc = readRead(volInfo, &testByte, 1);
     if(rc != 1)
         return false;
     
-    lseek(image, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     return (testByte == 0) ? false : true;
 }
@@ -457,7 +460,7 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
     unsigned lenExtent; /* parameter to readDirContents() */
     unsigned char lenFileId9660; /* also len joliet fileid (bytes) */
     int lenSU; /* calculated as recordLength - 33 - lenFileId9660 */
-    off_t origPos;
+    bk_off_t origPos;
     
     /* should anything fail, will still be safe to delete dir, this also
     * needs to be done before calling readDirContents() (now is good) */
@@ -468,23 +471,23 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
     
     maybeUpdateProgress(volInfo);
     
-    rc = read(volInfo->imageForReading, &recordLength, 1);
+    rc = readRead(volInfo, &recordLength, 1);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
-    lseek(volInfo->imageForReading, 1, SEEK_CUR);
+    readSeekSet(volInfo, 1, SEEK_CUR);
     
-    rc = read733(volInfo->imageForReading, &locExtent);
+    rc = read733(volInfo, &locExtent);
     if(rc != 8)
         return BKERROR_READ_GENERIC;
     
-    rc = read733(volInfo->imageForReading, &lenExtent);
+    rc = read733(volInfo, &lenExtent);
     if(rc != 8)
         return BKERROR_READ_GENERIC;
     
-    lseek(volInfo->imageForReading, 14, SEEK_CUR);
+    readSeekSet(volInfo, 14, SEEK_CUR);
     
-    rc = read(volInfo->imageForReading, &lenFileId9660, 1);
+    rc = readRead(volInfo, &lenFileId9660, 1);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
@@ -495,23 +498,23 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
     /* READ directory name */
     if(volInfo->rootRead)
     {
-        off_t posBeforeName = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+        bk_off_t posBeforeName = readSeekTell(volInfo);
         
-        rc = read(volInfo->imageForReading, BK_BASE_PTR(dir)->name, lenFileId9660);
+        rc = readRead(volInfo, BK_BASE_PTR(dir)->name, lenFileId9660);
         if(rc != lenFileId9660)
             return BKERROR_READ_GENERIC;
         BK_BASE_PTR(dir)->name[lenFileId9660] = '\0';
         
         /* record 9660 name for writing later */
-        strncpy(BK_BASE_PTR(dir)->original9660name, BK_BASE_PTR(dir)->name, 12);
-        BK_BASE_PTR(dir)->original9660name[12] = '\0';
+        strncpy(BK_BASE_PTR(dir)->original9660name, BK_BASE_PTR(dir)->name, 14);
+        BK_BASE_PTR(dir)->original9660name[14] = '\0';
         
         /* skip padding field if it's there */
         if(lenFileId9660 % 2 == 0)
-            lseek(volInfo->imageForReading, 1, SEEK_CUR);
+            readSeekSet(volInfo, 1, SEEK_CUR);
         
         if(filenameType != FNTYPE_9660)
-            lseek(volInfo->imageForReading, posBeforeName, SEEK_SET);
+            readSeekSet(volInfo, posBeforeName, SEEK_SET);
     }
     
     if(filenameType == FNTYPE_JOLIET)
@@ -527,7 +530,7 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
             if(lenFileId9660 % 2 != 0)
                 return BKERROR_INVALID_UCS2;
             
-            rc = read(volInfo->imageForReading, nameAsOnDisk, lenFileId9660);
+            rc = readRead(volInfo, nameAsOnDisk, lenFileId9660);
             if(rc != lenFileId9660)
                 return BKERROR_READ_GENERIC;
             
@@ -543,7 +546,7 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
             
             /* padding field */
             if(lenFileId9660 % 2 == 0)
-                lseek(volInfo->imageForReading, 1, SEEK_CUR);
+                readSeekSet(volInfo, 1, SEEK_CUR);
         }
     }
     else if(filenameType == FNTYPE_ROCKRIDGE)
@@ -551,10 +554,10 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
         if(volInfo->rootRead)
         {
             /* skip 9660 filename */
-            lseek(volInfo->imageForReading, lenFileId9660, SEEK_CUR);
+            readSeekSet(volInfo, lenFileId9660, SEEK_CUR);
             /* skip padding field */
             if(lenFileId9660 % 2 == 0)
-                lseek(volInfo->imageForReading, 1, SEEK_CUR);
+                readSeekSet(volInfo, 1, SEEK_CUR);
             
             rc = readRockridgeFilename(volInfo, BK_BASE_PTR(dir)->name, lenSU, 0);
             if(rc < 0)
@@ -571,25 +574,25 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
         {
             unsigned char realRootRecordLen;
             
-            origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+            origPos = readSeekTell(volInfo);
             
             /* go to real root record */
-            lseek(volInfo->imageForReading, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
+            readSeekSet(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
             
             /* read record length */
-            read(volInfo->imageForReading, &realRootRecordLen, 1);
+            readRead(volInfo, &realRootRecordLen, 1);
             if(rc != 1)
                 return BKERROR_READ_GENERIC;
             
             /* go to sys use fields */
-            lseek(volInfo->imageForReading, 33, SEEK_CUR);
+            readSeekSet(volInfo, 33, SEEK_CUR);
             
             rc = readPosixFileMode(volInfo, &(BK_BASE_PTR(dir)->posixFileMode), realRootRecordLen - 34);
             if(rc <= 0)
                 return rc;
             
             /* return */
-            lseek(volInfo->imageForReading, origPos, SEEK_SET);
+            readSeekSet(volInfo, origPos, SEEK_SET);
         }
         else
         {
@@ -604,11 +607,11 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
         BK_BASE_PTR(dir)->posixFileMode = volInfo->posixDirDefaults;
     }
     
-    lseek(volInfo->imageForReading, lenSU, SEEK_CUR);
+    readSeekSet(volInfo, lenSU, SEEK_CUR);
     
-    origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
     
-    lseek(volInfo->imageForReading, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
+    readSeekSet(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
     
     volInfo->rootRead = true;
     
@@ -616,7 +619,7 @@ int readDir(VolInfo* volInfo, BkDir* dir, int filenameType,
     if(rc < 0)
         return rc;
     
-    lseek(volInfo->imageForReading, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     return recordLength;
 }
@@ -630,16 +633,16 @@ int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size,
                     int filenameType, bool keepPosixPermissions)
 {
     int rc;
-    int bytesRead = 0;
-    int childrenBytesRead;
+    unsigned bytesRead = 0;
+    unsigned childrenBytesRead;
     BkFileBase** nextChild; /* pointer to pointer to modify pointer :) */
     
     /* skip self and parent */
-    rc = skipDR(volInfo->imageForReading);
+    rc = skipDR(volInfo);
     if(rc <= 0)
         return rc;
     bytesRead += rc;
-    rc = skipDR(volInfo->imageForReading);
+    rc = skipDR(volInfo);
     if(rc <= 0)
         return rc;
     bytesRead += rc;
@@ -648,19 +651,19 @@ int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size,
     childrenBytesRead = 0;
     while(childrenBytesRead + bytesRead < size)
     {
-        if(haveNextRecordInSector(volInfo->imageForReading))
+        if(haveNextRecordInSector(volInfo))
         /* read it */
         {
             int recordLength;
             
-            if( dirDrFollows(volInfo->imageForReading) )
+            if( dirDrFollows(volInfo) )
             /* directory descriptor record */
             {
                 *nextChild = malloc(sizeof(BkDir));
                 if(*nextChild == NULL)
                     return BKERROR_OUT_OF_MEMORY;
                 
-                bzero(*nextChild, sizeof(BkDir));
+                memset(*nextChild, 0, sizeof(BkDir));
                 
                 recordLength = readDir(volInfo, BK_DIR_PTR(*nextChild), 
                                        filenameType, keepPosixPermissions);
@@ -677,7 +680,7 @@ int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size,
                 if(*nextChild == NULL)
                     return BKERROR_OUT_OF_MEMORY;
                 
-                bzero(*nextChild, sizeof(BkFile));
+                memset(*nextChild, 0, sizeof(BkFile));
                 
                 recordLength = readFileInfo(volInfo, BK_FILE_PTR(*nextChild), 
                                             filenameType, keepPosixPermissions, 
@@ -703,19 +706,19 @@ int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size,
         *  sector btw) or get to the end of data (dir->self.dataLength) */
         {
             char testByte;
-            off_t origPos;
+            bk_off_t origPos;
             
             do
             {
-                origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+                origPos = readSeekTell(volInfo);
                 
-                rc = read(volInfo->imageForReading, &testByte, 1);
+                rc = readRead(volInfo, &testByte, 1);
                 if(rc != 1)
                     return BKERROR_READ_GENERIC;
                 
                 if(testByte != 0)
                 {
-                    lseek(volInfo->imageForReading, origPos, SEEK_SET);
+                    readSeekSet(volInfo, origPos, SEEK_SET);
                     break;
                 }
                 
@@ -741,6 +744,8 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
     unsigned lenExtent; /* in bytes */
     unsigned char lenFileId9660; /* also len joliet fileid (bytes) */
     int lenSU; /* calculated as recordLength - 33 - lenFileId9660 */
+    bk_off_t posBeforeName;
+    char nameAsOnDisk[UCHAR_MAX + 1];
     
     /* so if anything failes it's still safe to delete file */
     file->pathAndName = NULL;
@@ -752,17 +757,17 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
     
     *specialFile = NULL;
     
-    rc = read(volInfo->imageForReading, &recordLength, 1);
+    rc = readRead(volInfo, &recordLength, 1);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
-    lseek(volInfo->imageForReading, 1, SEEK_CUR);
+    readSeekSet(volInfo, 1, SEEK_CUR);
     
-    rc = read733(volInfo->imageForReading, &locExtent);
+    rc = read733(volInfo, &locExtent);
     if(rc != 8)
         return BKERROR_READ_GENERIC;
     
-    rc = read733(volInfo->imageForReading, &lenExtent);
+    rc = read733(volInfo, &lenExtent);
     if(rc != 8)
         return BKERROR_READ_GENERIC;
     
@@ -782,9 +787,9 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
         volInfo->bootRecordOnImage = file;
     }
     
-    lseek(volInfo->imageForReading, 14, SEEK_CUR);
+    readSeekSet(volInfo, 14, SEEK_CUR);
     
-    rc = read(volInfo->imageForReading, &lenFileId9660, 1);
+    rc = readRead(volInfo, &lenFileId9660, 1);
     if(rc != 1)
         return BKERROR_READ_GENERIC;
     
@@ -793,15 +798,14 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
         lenSU -= 1;
     
     /* READ 9660 name */
-    off_t posBeforeName = lseek(volInfo->imageForReading, 0, SEEK_CUR);
-    char nameAsOnDisk[UCHAR_MAX + 1];
+    posBeforeName = readSeekTell(volInfo);
     
-    rc = read(volInfo->imageForReading, nameAsOnDisk, lenFileId9660);
+    rc = readRead(volInfo, nameAsOnDisk, lenFileId9660);
     if(rc != lenFileId9660)
         return BKERROR_READ_GENERIC;
     nameAsOnDisk[lenFileId9660] = '\0';
     
-    //removeCrapFromFilename(nameAsOnDisk, lenFileId9660);
+    /* removeCrapFromFilename(nameAsOnDisk, lenFileId9660); */
     
     strncpy(BK_BASE_PTR(file)->name, nameAsOnDisk, NCHARS_FILE_ID_MAX_STORE - 1);
     BK_BASE_PTR(file)->name[NCHARS_FILE_ID_MAX_STORE - 1] = '\0';
@@ -812,10 +816,10 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
     
     /* padding field */
     if(lenFileId9660 % 2 == 0)
-        lseek(volInfo->imageForReading, 1, SEEK_CUR);
+        readSeekSet(volInfo, 1, SEEK_CUR);
     
     if(filenameType != FNTYPE_9660)
-            lseek(volInfo->imageForReading, posBeforeName, SEEK_SET);
+            readSeekSet(volInfo, posBeforeName, SEEK_SET);
     /* END READ 9660 name */
     
     if(filenameType == FNTYPE_JOLIET)
@@ -828,7 +832,7 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
         if(lenFileId9660 % 2 != 0)
             return BKERROR_INVALID_UCS2;
         
-        rc = read(volInfo->imageForReading, nameAsOnDisk, lenFileId9660);
+        rc = readRead(volInfo, nameAsOnDisk, lenFileId9660);
         if(rc != lenFileId9660)
             return BKERROR_READ_GENERIC;
         
@@ -848,15 +852,15 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
         
         /* padding field */
         if(lenFileId9660 % 2 == 0)
-            lseek(volInfo->imageForReading, 1, SEEK_CUR);
+            readSeekSet(volInfo, 1, SEEK_CUR);
     }
     else if(filenameType == FNTYPE_ROCKRIDGE)
     {
         /* skip 9660 filename */
-        lseek(volInfo->imageForReading, lenFileId9660, SEEK_CUR);
+        readSeekSet(volInfo, lenFileId9660, SEEK_CUR);
         /* skip padding field */
         if(lenFileId9660 % 2 == 0)
-            lseek(volInfo->imageForReading, 1, SEEK_CUR);
+            readSeekSet(volInfo, 1, SEEK_CUR);
         
         rc = readRockridgeFilename(volInfo, BK_BASE_PTR(file)->name, lenSU, 0);
         if(rc < 0)
@@ -884,6 +888,7 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
     /* the file is actually a symbolic link */
     {
         strcpy((*specialFile)->name, BK_BASE_PTR(file)->name);
+        strcpy((*specialFile)->original9660name, BK_BASE_PTR(file)->original9660name);
         /* apparently permissions for symbolic links are never used */
         (*specialFile)->posixFileMode = 0120777;
     }
@@ -909,7 +914,7 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
         file->location = newLink;
     }
     
-    lseek(volInfo->imageForReading, lenSU, SEEK_CUR);
+    readSeekSet(volInfo, lenSU, SEEK_CUR);
     
     file->onImage = true;
     file->position = locExtent * NBYTES_LOGICAL_BLOCK;
@@ -922,9 +927,9 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
 * readPosixFileMode()
 * looks for the PX system use field and gets the permissions field out of it
 * */
-int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, unsigned lenSU)
+int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, int lenSU)
 {
-    off_t origPos;
+    bk_off_t origPos;
     unsigned char* suFields;
     int rc;
     bool foundPosix;
@@ -938,12 +943,12 @@ int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, unsigned lenSU)
     if(suFields == NULL)
         return BKERROR_OUT_OF_MEMORY;
     
-    origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
+    
+    rc = readRead(volInfo, suFields, lenSU);
     
-    rc = read(volInfo->imageForReading, suFields, lenSU);
     if(rc != lenSU)
         return BKERROR_READ_GENERIC;
-    
     count = 0;
     foundPosix = false;
     foundCE = false;
@@ -975,7 +980,7 @@ int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, unsigned lenSU)
     }
     
     free(suFields);
-    lseek(volInfo->imageForReading, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     if(!foundPosix)
     {
@@ -983,11 +988,11 @@ int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, unsigned lenSU)
             return BKERROR_NO_POSIX_PRESENT;
         else
         {
-            lseek(volInfo->imageForReading, logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + 
-                  offsetInLogicalBlockOfCE, SEEK_SET);
+            readSeekSet(volInfo, logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + 
+                        offsetInLogicalBlockOfCE, SEEK_SET);
             rc = readPosixFileMode(volInfo, posixFileMode, lengthOfCE);
             
-            lseek(volInfo->imageForReading, origPos, SEEK_SET);
+            readSeekSet(volInfo, origPos, SEEK_SET);
             
             return rc;
         }
@@ -1004,10 +1009,10 @@ int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, unsigned lenSU)
 * this directory record, the function returns a failure.
 * Leaves the file pointer where it was.
 */
-int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU, 
+int readRockridgeFilename(VolInfo* volInfo, char* dest, int lenSU, 
                           unsigned numCharsReadAlready)
 {
-    off_t origPos;
+    bk_off_t origPos;
     unsigned char* suFields;
     int rc;
     int count;
@@ -1024,9 +1029,9 @@ int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU,
     if(suFields == NULL)
         return BKERROR_OUT_OF_MEMORY;
     
-    origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
     
-    rc = read(volInfo->imageForReading, suFields, lenSU);
+    rc = readRead(volInfo, suFields, lenSU);
     if(rc != lenSU)
     {
         free(suFields);
@@ -1077,7 +1082,7 @@ int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU,
     }
     
     free(suFields);
-    lseek(volInfo->imageForReading, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     if( !foundName || (foundName && nameContinues) )
     {
@@ -1085,12 +1090,12 @@ int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU,
             return BKERROR_RR_FILENAME_MISSING;
         else
         {
-            lseek(volInfo->imageForReading, 
-                  logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + offsetInLogicalBlockOfCE, 
-                  SEEK_SET);
+            readSeekSet(volInfo, 
+                        logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + offsetInLogicalBlockOfCE, 
+                        SEEK_SET);
             rc = readRockridgeFilename(volInfo, dest, lengthOfCE, numCharsReadAlready);
             
-            lseek(volInfo->imageForReading, origPos, SEEK_SET);
+            readSeekSet(volInfo, origPos, SEEK_SET);
             
             return rc;
         }
@@ -1100,9 +1105,9 @@ int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU,
 }
 
 /* if no SL record is found does not return failure */
-int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, unsigned lenSU)
+int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, int lenSU)
 {
-    off_t origPos;
+    bk_off_t origPos;
     unsigned char* suFields;
     int rc;
     int count;
@@ -1112,9 +1117,9 @@ int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, unsigned lenSU)
     if(suFields == NULL)
         return BKERROR_OUT_OF_MEMORY;
     
-    origPos = lseek(volInfo->imageForReading, 0, SEEK_CUR);
+    origPos = readSeekTell(volInfo);
     
-    rc = read(volInfo->imageForReading, suFields, lenSU);
+    rc = readRead(volInfo, suFields, lenSU);
     if(rc != lenSU)
     {
         free(suFields);
@@ -1131,13 +1136,13 @@ int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, unsigned lenSU)
         
         if(suFields[count] == 'S' && suFields[count + 1] == 'L')
         {
-            int numCharsUsed; /* in dest->target, not including '\0' */
+            size_t numCharsUsed; /* in dest->target, not including '\0' */
             
             *dest = malloc(sizeof(BkSymLink));
             if(*dest == NULL)
                 return BKERROR_OUT_OF_MEMORY;
             
-            bzero(*dest, sizeof(BkSymLink));
+            memset(*dest, 0, sizeof(BkSymLink));
             
             numCharsUsed = 0;
             (*dest)->target[0] = '\0';
@@ -1194,7 +1199,7 @@ int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, unsigned lenSU)
     }
     
     free(suFields);
-    lseek(volInfo->imageForReading, origPos, SEEK_SET);
+    readSeekSet(volInfo, origPos, SEEK_SET);
     
     return 1;
 }
@@ -1226,16 +1231,16 @@ void removeCrapFromFilename(char* filename, int length)
 * skipDR()
 * Seek past a directory entry. Good for skipping "." and ".."
 * */
-int skipDR(int image)
+int skipDR(VolInfo* volInfo)
 {
     unsigned char dRLen;
     int rc;
     
-    rc = read711(image, &dRLen);
+    rc = read711(volInfo, &dRLen);
     if(rc <= 0)
         return BKERROR_READ_GENERIC;
     
-    lseek(image, dRLen - 1, SEEK_CUR);
+    readSeekSet(volInfo, dRLen - 1, SEEK_CUR);
     
     return dRLen;
 }
@@ -1247,10 +1252,13 @@ int skipDR(int image)
 * */
 void stripSpacesFromEndOfString(char* str)
 {
-    int count;
+    size_t count;
     
-    for(count = strlen(str) - 1; count >= 0 && str[count] == ' '; count--)
+    for(count = strlen(str) - 1; str[count] == ' '; count--)
     {
         str[count] = '\0';
+
+        if(count == 0) /* unsigned */
+            break;
     }
 }
