@@ -1,7 +1,7 @@
-int appendStringIfHaveRoom(char* dest, char* src, int destMaxLen, 
-                           int destCharsAlreadyUsed, int maxSrcLen);
-bool dirDrFollows(int image);
-bool haveNextRecordInSector(int image);
+size_t appendStringIfHaveRoom(char* dest, const char* src, size_t destMaxLen, 
+                              size_t destCharsAlreadyUsed, int maxSrcLen);
+bool dirDrFollows(VolInfo* volInfo);
+bool haveNextRecordInSector(VolInfo* volInfo);
 int readDir(VolInfo* volInfo, BkDir* dir, int filenameType, 
             bool keepPosixPermissions);
 int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size, 
@@ -10,10 +10,10 @@ int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType,
                  bool keepPosixPermissions, BkFileBase** specialFile);
 unsigned char readNextRecordLen(int image);
 int readPosixFileMode(VolInfo* volInfo, unsigned* posixPermissions, 
-                         unsigned lenSU);
-int readRockridgeFilename(VolInfo* volInfo, char* dest, unsigned lenSU, 
+                      int lenSU);
+int readRockridgeFilename(VolInfo* volInfo, char* dest, int lenSU, 
                           unsigned numCharsReadAlready);
-int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, unsigned lenSU);
+int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, int lenSU);
 void removeCrapFromFilename(char* filename, int length);
-int skipDR(int image);
+int skipDR(VolInfo* volInfo);
 void stripSpacesFromEndOfString(char* str);
